import {
  NativeModules,
  NativeEventEmitter,
  Linking,
  Platform,
} from "react-native";

const { RNDisplay, RNAutoStartHelper } = NativeModules;
const SystemSettingNative = NativeModules.RNSystemSetting;

export { RNDisplay, RNAutoStartHelper, RNSystemSetting };

const isAndroid = Platform.OS === "android";

const isIOS = Platform.OS === "ios";

const SCREEN_BRIGHTNESS_MODE_UNKNOW = -1;
const SCREEN_BRIGHTNESS_MODE_MANUAL = 0;
const SCREEN_BRIGHTNESS_MODE_AUTOMATIC = 1;

const eventEmitter = new NativeEventEmitter(SystemSettingNative);

export default class RNSystemSetting {
  static saveBrightnessVal = -1;
  static saveScreenModeVal = SCREEN_BRIGHTNESS_MODE_AUTOMATIC;

  /**
   * @deprecated
   */
  static setAppStore() {
    console.warn("You don't need call setAppStore() anymore since V1.7.0");
  }

  static async getBrightness() {
    return await SystemSettingNative.getBrightness();
  }

  static async setBrightness(val) {
    try {
      await SystemSettingNative.setBrightness(val);
      return true;
    } catch (e) {
      return false;
    }
  }

  static async setBrightnessForce(val) {
    if (isAndroid) {
      const success = await RNSystemSetting.setScreenMode(
        SCREEN_BRIGHTNESS_MODE_MANUAL
      );
      if (!success) {
        return false;
      }
    }
    return await RNSystemSetting.setBrightness(val);
  }

  static setAppBrightness(val) {
    if (isAndroid) {
      SystemSettingNative.setAppBrightness(val);
    } else {
      RNSystemSetting.setBrightness(val);
    }
    return true;
  }

  static async getAppBrightness() {
    if (isAndroid) {
      return SystemSettingNative.getAppBrightness();
    } else {
      return RNSystemSetting.getBrightness();
    }
  }

  static grantWriteSettingPremission() {
    if (isAndroid) {
      SystemSettingNative.openWriteSetting();
    }
  }

  static async getScreenMode() {
    if (isAndroid) {
      return await SystemSettingNative.getScreenMode();
    }
    return -1; // cannot get iOS screen mode
  }

  static async setScreenMode(val) {
    if (isAndroid) {
      try {
        await SystemSettingNative.setScreenMode(val);
      } catch (e) {
        return false;
      }
    }
    return true;
  }

  static async saveBrightness() {
    RNSystemSetting.saveBrightnessVal = await RNSystemSetting.getBrightness();
    RNSystemSetting.saveScreenModeVal = await RNSystemSetting.getScreenMode();
  }

  static restoreBrightness() {
    if (RNSystemSetting.saveBrightnessVal == -1) {
      console.warn("you should call saveBrightness() at least once");
    } else {
      RNSystemSetting.setBrightness(RNSystemSetting.saveBrightnessVal);
      RNSystemSetting.setScreenMode(RNSystemSetting.saveScreenModeVal);
    }
    return RNSystemSetting.saveBrightnessVal;
  }

  static async getVolume(type = "music") {
    return await SystemSettingNative.getVolume(type);
  }

  static setVolume(val, config = {}) {
    if (typeof config === "string") {
      console.log(
        "setVolume(val, type) is deprecated since 1.2.2, use setVolume(val, config) instead"
      );
      config = { type: config };
    }
    config = Object.assign(
      {
        playSound: false,
        type: "music",
        showUI: false,
      },
      config
    );
    SystemSettingNative.setVolume(val, config);
  }

  static addVolumeListener(callback) {
    return eventEmitter.addListener("EventVolume", callback);
  }

  static removeVolumeListener(listener) {
    listener && listener.remove();
  }

  static async isWifiEnabled() {
    const result = await SystemSettingNative.isWifiEnabled();
    return result > 0;
  }

  static switchWifiSilence(complete) {
    if (isAndroid) {
      RNSystemSetting.listenEvent(complete);
      SystemSettingNative.switchWifiSilence();
    } else {
      RNSystemSetting.switchWifi(complete);
    }
  }

  static switchWifi(complete) {
    RNSystemSetting.listenEvent(complete);
    SystemSettingNative.switchWifi();
  }

  static async isLocationEnabled() {
    return await SystemSettingNative.isLocationEnabled();
  }

  static async getLocationMode() {
    if (isAndroid) {
      return await SystemSettingNative.getLocationMode();
    } else {
      return (await RNSystemSetting.isLocationEnabled()) ? 1 : 0;
    }
  }

  static switchLocation(complete) {
    RNSystemSetting.listenEvent(complete);
    SystemSettingNative.switchLocation();
  }

  static async isBluetoothEnabled() {
    return await SystemSettingNative.isBluetoothEnabled();
  }

  static switchBluetooth(complete) {
    RNSystemSetting.listenEvent(complete);
    SystemSettingNative.switchBluetooth();
  }

  static switchBluetoothSilence(complete) {
    if (isAndroid) {
      RNSystemSetting.listenEvent(complete);
      SystemSettingNative.switchBluetoothSilence();
    } else {
      SystemSettingNative.switchBluetooth(complete);
    }
  }

  static async isAirplaneEnabled() {
    return await SystemSettingNative.isAirplaneEnabled();
  }

  static switchAirplane(complete) {
    RNSystemSetting.listenEvent(complete);
    SystemSettingNative.switchAirplane();
  }

  static async openAppSystemSettings() {
    switch (Platform.OS) {
      case "ios": {
        const settingsLink = "app-settings:";
        const supported = await Linking.canOpenURL(settingsLink);
        if (supported) await Linking.openURL(settingsLink);
        break;
      }
      case "android":
        await SystemSettingNative.openAppSystemSettings();
        break;
      default:
        throw new Error("unknown platform");
        break;
    }
  }

  static async addBluetoothListener(callback) {
    return await RNSystemSetting._addListener(
      false,
      "bluetooth",
      "EventBluetoothChange",
      callback
    );
  }

  static async addWifiListener(callback) {
    return await RNSystemSetting._addListener(
      true,
      "wifi",
      "EventWifiChange",
      callback
    );
  }

  static async addLocationListener(callback) {
    return await RNSystemSetting._addListener(
      true,
      "location",
      "EventLocationChange",
      callback
    );
  }

  static async addLocationModeListener(callback) {
    return await RNSystemSetting._addListener(
      true,
      "locationMode",
      "EventLocationModeChange",
      callback
    );
  }

  static async addAirplaneListener(callback) {
    return await RNSystemSetting._addListener(
      true,
      "airplane",
      "EventAirplaneChange",
      callback
    );
  }

  static async _addListener(androidOnly, type, eventName, callback) {
    if (!androidOnly || isAndroid) {
      const result = await RNSystemSetting._activeListener(type);
      if (result) {
        return eventEmitter.addListener(eventName, callback);
      }
    }
    return null;
  }

  static async _activeListener(name) {
    try {
      await SystemSettingNative.activeListener(name);
    } catch (e) {
      console.warn(e.message);
      return false;
    }
    return true;
  }

  static removeListener(listener) {
    listener && listener.remove();
  }

  static listenEvent(complete) {
    const listener = eventEmitter.addListener("EventEnterForeground", () => {
      listener.remove();
      complete();
    });
  }
}
